module( ..., package.seeall )

local aperture = require( 'saotrace.aperture.mock' )

local ideal = require('saotrace.aperture.ideal').create_baffles

local table = require( 'table' )
local os = require( 'os' )
local srcdir = os.getenv('srcdir') or '.'
local db = table.concat( { srcdir, 'data', 'geometry.rdb'}, '/' )

local giter = require('saotrace.geometry').read{ file = db, mode = 'iterator' }

local optic = {}

for geo in giter do

   if not optic[geo.optic] then
      optic[geo.optic] = {}
   end

   optic[geo.optic][geo.shell] = geo

end

setup = _G.setup

function test_outermost( )

   ideal{ shell = 1,
	  optic = 'p',
	  name = 'precollimator',
	  cols = {
	     x = 'x0',
	     y = 'y0',
	     z = 'z_f',
	     r = 'rho_f',
	  },
	  db = db
       }

   -- test for correct radii of generated circles.
   -- first is p3
   local stack = aperture._get_stack()

   assert_equal( optic.p[3].rho_f, stack[4][2], 'p3' )

   -- next is p1

   assert_equal( optic.p[1].rho_f, stack[8][2], 'p1' )
end

function test_middle( )

   ideal{ shell = 3,
	  optic = 'p',
	  name = 'precollimator',
	  cols = {
	     x = 'x0',
	     y = 'y0',
	     z = 'z_f',
	     r = 'rho_f',
	  },
	  db = db
       }

   -- test for correct radii of generated circles.
   -- first is p4
   local stack = aperture._get_stack()

   assert_equal( optic.p[4].rho_f, stack[4][2], 'p3' )

   -- next is p3

   assert_equal( optic.p[3].rho_f, stack[8][2], 'p1' )
end

function test_inner_disk( )

   ideal{ shell = 6,
	  optic = 'p',
	  name = 'precollimator',
	  cols = {
	     x = 'x0',
	     y = 'y0',
	     z = 'z_f',
	     r = 'rho_f',
	  },
	  db = db,
	  inner_disk = { z = 20,
			 r = 10 },
       }

   -- test for correct radii of generated circles.
   -- first is p3
   local stack = aperture._get_stack()

   assert_equal( 20, stack[3][4], 'p4-p6' )

   assert_equal( 10, stack[4][2], 'p4-p6' )

   -- next is p6

   assert_equal( optic.p[6].rho_f, stack[8][2], 'p6' )
end

function test_no_inner_disk( )

   ideal{ shell = 6,
	  optic = 'p',
	  name = 'precollimator',
	  cols = {
	     x = 'x0',
	     y = 'y0',
	     z = 'z_f',
	     r = 'rho_f',
	  },
	  db = db
       }

   -- test for correct radii of generated circles.
   -- first is p3
   local stack = aperture._get_stack()

   assert_equal( optic.p[6].rho_f - (optic.p[4].rho_f - optic.p[6].rho_f), stack[4][2], 'p4-p6' )

   -- next is p6

   assert_equal( optic.p[6].rho_f, stack[8][2], 'p6' )
end
